@@ -242,7 +242,12 @@ class AIOKerberosClient:
 					logger.debug('Server supports encryption type %s with salt %s' % (EncryptionType(enc_info['etype']).name, enc_info['salt']))
 		
 		preferred_enc_type = self.usercreds.get_preferred_enctype(supp_enc_methods)
-		self.server_salt = supp_enc_methods[preferred_enc_type].encode() #enc_info['salt'].encode() 
+		if preferred_enc_type not in supp_enc_methods:
+			raise Exception('Preferred enc type not in supported enctypes')
+		salt = supp_enc_methods[preferred_enc_type]
+		if salt is not None:
+			salt = salt.encode()
+		self.server_salt = salt #enc_info['salt'].encode()
 		return preferred_enc_type
 
 
