@@ -9,15 +9,15 @@ class KerberosError(Exception):
 	def __init__(self, krb_err_msg, extra_msg = ''):
 		self.krb_err_msg = krb_err_msg.native
 		self.errorcode = KerberosErrorCode.ERR_NOT_FOUND
-		self.errormsg = 'Error code not found! Err code: %s' % self.krb_err_msg['error-code']
+		self.errormsg = 'Error message not found! Err code: %s' % self.krb_err_msg['error-code']
 		try:
 			self.errorcode = KerberosErrorCode(self.krb_err_msg['error-code'])
-			self.errormsg = KerberosErrorMessage[self.errorcode.name]
+			self.errormsg = KerberosErrorMessage[self.errorcode.name].value
 		except:
 			pass
 		self.extra_msg = extra_msg
 		
-		super(Exception, self).__init__('%s Error Code: %d Reason: %s ' % (extra_msg, self.errorcode.value, self.errormsg.value))
+		super(Exception, self).__init__('%s Error Name: %s Detail: "%s" ' % (extra_msg, self.errorcode.name, self.errormsg))
 	
 		
 
@@ -155,4 +155,4 @@ class KerberosErrorMessage(enum.Enum):
 	KRB_AP_ERR_NO_TGT  ='No TGT was presented or available'
 	KDC_ERR_WRONG_REALM = 'Incorrect domain or principal'
 	KRB_AP_ERR_IAKERB_KDC_NOT_FOUND = 'The IAKERB proxy could not find a KDC'
-	KRB_AP_ERR_IAKERB_KDC_NO_RESPONSE = 'The KDC did not respond to the IAKERB proxy'
\ No newline at end of file
+	KRB_AP_ERR_IAKERB_KDC_NO_RESPONSE = 'The KDC did not respond to the IAKERB proxy'
